/***********************************************************************
 * $Id:: nanderase.c 7828 2011-08-04 10:22:46Z ing03005               $
 *
 * Project: Simple NAND erase program
 *
 * Description:
 *     Erases the entire NAND flash.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

#include "lpc_types.h"
#include "lpc32xx_slcnand_driver.h"
#include "misc_config.h"
#include "common_funcs.h"

/***********************************************************************
 *
 * Function: print_uint
 *
 * Purpose: Prints an unsigned integer to UART
 *
 * Processing:
 *     See function.
 *
 * Parameters: val --> Value to be printed
 *             rec --> Always be 0
 *
 * Outputs: None
 *
 * Returns: None
 *
 * Notes: None
 *
 **********************************************************************/
void print_uint (unsigned long val, int rec)
{
	if (!val)
	{
		if (!rec)
			uart_output((UNS_8 *)"0");
		return;
	}
	print_uint(val/10, 1);
	*(UNS_8 *)&rec = '0' + (val - (val/10) * 10);
	*(((UNS_8 *)&rec) + 1) = 0;
	uart_output((UNS_8 *)&rec);
}

/***********************************************************************
 *
 * Function: c_entry
 *
 * Purpose: Application entry point from the startup code
 *
 * Processing:
 *     See function.
 *
 * Parameters: None
 *
 * Outputs: None
 *
 * Returns: None
 *
 * Notes: None
 *
 **********************************************************************/
void c_entry(void)
{
  UNS_32 i;

  INT_32 nand;
  NAND_GEOM_T geom;

  uart_output_init();

  nand = slcnand_board_init();
  /* Init NAND controller */
  if (nand == 0)
  {
   	uart_output((UNS_8*)"Cannot initialize NAND device\r\n");
	while (1);
  }

  slcnand_get_geom(nand, &geom);
  /* Disable write protect */
  nand_flash_wp_disable();

  uart_output((UNS_8*)"\r\nWARNING: This will erase all blocks "
		  "including factory bad blocks \r\nPress any key to continue...");
  while(uart_input(&i, 1) < 1);
  uart_output((UNS_8*)"\r\nFormatting NAND Flash...\r\n");

  /* Erase 25 blocks */
  for (i = 0; i < geom.num_blocks; i++)
  {
    if (slcnand_erase_block(nand, i) == _ERROR)
    {
      /* Erase failure, mark the block as bad */
      uart_output((UNS_8*)"Failed to erase block: ");
	  print_uint((unsigned long)i, 0);
	  uart_output((UNS_8*)"\r\n");
    }
    else
    {
      char *spin = "|\0/\0-\0\\";
	  uart_output((UNS_8*)"\033[0G"); /* Reset cursor to column 0 */
	  uart_output((UNS_8*)spin + ((i & 0x3) << 1));
      uart_output((UNS_8*)" Erasing [");
	  print_uint((unsigned long)i + 1, 0);
      uart_output((UNS_8*)"/");
	  print_uint((unsigned long)geom.num_blocks, 0);
	  uart_output((UNS_8*)"] ");
    }
  }
  uart_output((UNS_8 *)"\r\nFormat Completed\r\n");

  /* Loop forever */
  while (1);
}
