/***********************************************************************
 * $Id:: kickstart_nand.c 7821 2011-08-03 17:11:40Z ing03005           $
 *
 * Project: Kickstart loader from large block NAND FLASH
 *
 * Description:
 *     This file contains a sample kickstart loader that can be used
 *     to load a larger stage 1 application from large block NAND
 *     FLASH.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

#include "lpc32xx_slcnand_driver.h"
#include "lpc32xx_dma_driver.h"
#include "lpc32xx_intc_driver.h"
#include "lpc_irq_fiq.h"
#include "startup.h"
#include "misc_config.h"

static DMA_BUFFER UNS_8 tmpbuff [2048+64];

/* Prototype for external IRQ handler */
void lpc32xx_irq_handler(void);
static INT_32 nand;
/***********************************************************************
 *
 * Function: c_entry
 *
 * Purpose: Application entry point from the startup code
 *
 * Processing:
 *     See function.
 *
 * Parameters: None
 *
 * Outputs: None
 *
 * Returns: Nothing
 *
 * Notes: None
 *
 **********************************************************************/
void c_entry(void) {
	UNS_8 *p8, ret;
	INT_32 toread, idx, blk, page, sector;
	PFV execa = (PFV) STAGE1_LOAD_ADDR;
	NAND_GEOM_T nandgeom;

	int_initialize(0xFFFFFFFF);

	/* Install standard IRQ dispatcher at ARM IRQ vector */
	int_install_arm_vec_handler(IRQ_VEC, (PFV) lpc32xx_irq_handler);
	enable_irq();

	nand = slcnand_board_init();
	/* Initialize NAND FLASH */
	if (nand == 0)
	{
		while (1);
	}

	slcnand_get_geom(nand, &nandgeom);
	/* Read data into memory */
	toread = STAGE1_LOAD_SIZE;
	blk = 1;
	page = 0;
	p8 = (UNS_8 *) STAGE1_LOAD_ADDR;
	while (toread > 0)
	{
		ret = slcnand_is_block_bad(nand, blk);
		if (ret == 0)
		{
			while(page < nandgeom.pages_per_block)
			{
				sector = blk * nandgeom.pages_per_block + page;
				slcnand_read_sector(nand, sector, tmpbuff,NULL);
				for (idx = 0; idx < nandgeom.data_bytes_per_page; idx++)
				{
					*p8 = tmpbuff [idx];
					p8++;
				}
				page++;
				toread = toread - nandgeom.data_bytes_per_page;
			}
			blk++;
			page = 0;
		}
		else
		{
			blk++;
		}
	}

	disable_irq();
#ifdef USE_MMU
	dcache_flush();
	dcache_inval();
	icache_inval();
#endif

	execa();
}
