######################################################################## 
# $Id:: kickstart_nand.mak 8091 2011-09-14 15:36:24Z ing03005          $
# 
# Project: Kickstart makefile for large block NAND
# 
######################################################################## 
# Software that is described herein is for illustrative purposes only  
# which provides customers with programming information regarding the  
# products. This software is supplied "AS IS" without any warranties.  
# NXP Semiconductors assumes no responsibility or liability for the 
# use of the software, conveys no license or title under any patent, 
# copyright, or mask work right to the product. NXP Semiconductors 
# reserves the right to make changes in the software without 
# notification. NXP Semiconductors also make no representation or 
# warranty that such application will be suitable for the specified 
# use without further testing or modification. 
########################################################################

########################################################################
#
# Pick up the default build rules 
#
########################################################################

COMMON_BASE := $(NXPMCU_SOFTWARE)/csps/$(CSP)/bsps/common
include $(NXPMCU_SOFTWARE)/makerule/$(CSP)/make.$(CSP).$(TOOL)
include $(COMMON_BASE)/startup/examples/buildfiles/make.env

########################################################################
# ARM Realview
########################################################################
ifeq ($(TOOL), rvw)

# Full board setup and MMU code
AFLAGS_MMU  := --predefine "USE_MMU SETL {TRUE}"
AFLAGS_INIT := --predefine "USE_BOARD_INIT SETL {TRUE}"
AFLAGS += --predefine "USE_ALL_STACKS SETL {TRUE}"
#AFLAGS += --predefine "RW_RELOC SETL {TRUE}"

# The kickstart loader loads and executes at address 0x0 in IRAM.
# Suitable for NAND FLASH and SPI boot modes
LDSCRIPT =$(COMMON_BASE)/startup/examples/buildfiles/ldscript_iram_rvw.ld
LDFLAGS = --remove
MAP = --map --info=totals,sizes,unused --symbols --list
endif

########################################################################
# Keil MDK justs uses the ARM Realview defines
########################################################################
ifeq ($(TOOL), keil)

# Full board setup and MMU code
AFLAGS_MMU  := --predefine "USE_MMU SETL {TRUE}"
AFLAGS_INIT := --predefine "USE_BOARD_INIT SETL {TRUE}"
AFLAGS += --predefine "USE_ALL_STACKS SETL {TRUE}"
#AFLAGS += --predefine "RW_RELOC SETL {TRUE}"

# The kickstart loader loads and executes at address 0x0 in IRAM.
# Suitable for NAND FLASH and SPI boot modes
LDSCRIPT =$(COMMON_BASE)/startup/examples/buildfiles/ldscript_iram_rvw.ld
LDFLAGS = --remove
MAP = --map --info=totals,sizes,unused --symbols --list
endif

########################################################################
# GNU
########################################################################
ifeq ($(TOOL), gnu)

# Full board setup and MMU code
AFLAGS_MMU  := --defsym USE_MMU=1
AFLAGS_INIT := --defsym USE_BOARD_INIT=1
AFLAGS += --defsym USE_ALL_STACKS=1
#AFLAGS += --defsym RW_RELOC=1

# The kickstart loader loads and executes at address 0x0 in IRAM.
# Suitable for NAND FLASH and SPI boot modes
LDSCRIPT =$(COMMON_BASE)/startup/examples/buildfiles/ldscript_iram_gnu.ld

endif

########################################################################
#
# Compiler/linker specific stuff
#
########################################################################

ifeq ($(TOOL), rvw)
MEXT        =.map
MAPFILE     =$(EXECNAME)
C_ENTRY     =
ENTRY       =
endif

ifeq ($(TOOL), keil)
MEXT        =.map
MAPFILE     =$(EXECNAME)
C_ENTRY     =
ENTRY       =
endif

ifeq ($(TOOL), gnu)
MEXT        =.map
MAPFILE     =$(EXECNAME)
C_ENTRY     =
ENTRY       =
endif

VPATH += $(COMMON_BASE)/startup/examples/kickstart $(EXTRA_VPATH)
########################################################################
#
# Pick up the assembler and C source files in the directory  
#
########################################################################
include $(NXPMCU_SOFTWARE)/makerule/common/make.rules.ftypes

########################################################################
#
# Kickstart code object files 
#
########################################################################
OBJS += kickstart_nand.o $(EXTRA_OBJS)
OBJS += cache_support.o startup_entry.o

default: $(OBJS) lpc_libs elf bin hex

elf:
	$(LD) $(OBJS) $(LDFLAGS) $(LK) $(SCAN) $(MAP) $(MAPFILE)$(MEXT) \
	$(LDESC) $(LDSCRIPT) $(ENTRY) $(C_ENTRY) -o $(EXECNAME)$(EXT)

ifeq ($(TOOL), rvw)
bin: 
	$(ELFTOBIN) $(EXECNAME)$(EXT) --output $(EXECNAME).bin

hex:
	$(ELFTOREC) $(EXECNAME)$(EXT) --output $(EXECNAME)$(REC)
endif

ifeq ($(TOOL), keil)
bin: 
	$(ELFTOBIN) $(EXECNAME)$(EXT) --output $(EXECNAME).bin

hex:
	$(ELFTOREC) $(EXECNAME)$(EXT) --output $(EXECNAME)$(REC)
endif

ifeq ($(TOOL), gnu)
bin: 
	$(ELFTOBIN) $(EXECNAME)$(EXT) $(EXECNAME).bin

hex:
	$(ELFTOREC) $(EXECNAME)$(EXT) $(EXECNAME)$(REC)
endif

########################################################################
#
# Pick up the compiler and assembler rules
#
########################################################################

include $(NXPMCU_SOFTWARE)/makerule/common/make.rules.build

.PHONY: debug bin

