/***********************************************************************
 * $Id:: board_nand_ids.c 7809 2011-08-02 04:25:41Z ing03005 $
 *
 * Project: NAND Devices list for EA3250 board
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/
#include "lpc32xx_slcnand_driver.h"
#include "lpc32xx_mlcnand_driver.h"
#include "lpc_nandflash_params.h"
#include "board_config.h"

/* Timing parameter */
#define EA3250_SLC_TIMING \
		SLCTAC_WDR(SLC_NAND_W_RDY) | \
		SLCTAC_WWIDTH(SLC_NAND_W_WIDTH) | \
		SLCTAC_WHOLD(SLC_NAND_W_HOLD) | \
		SLCTAC_WSETUP(SLC_NAND_W_SETUP) | \
		SLCTAC_RDR(SLC_NAND_R_RDY) | \
		SLCTAC_RWIDTH(SLC_NAND_R_WIDTH) | \
		SLCTAC_RHOLD(SLC_NAND_R_HOLD) | \
		SLCTAC_RSETUP(SLC_NAND_R_SETUP)

#define EA3250_MLC_TIMING MLC_LOAD_TCEA(MLC_TCEA_TIME) | \
		MLC_LOAD_TWBTRB(MLC_TWBTRB_TIME) | \
        MLC_LOAD_TRHZ(MLC_TRHZ_TIME) | \
		MLC_LOAD_TREH(MLC_TREH_TIME) | \
        MLC_LOAD_TRP(MLC_TRP_TIME) | \
		MLC_LOAD_TWH(MLC_TWH_TIME) | \
		MLC_LOAD_TWP(MLC_TWP_TIME)

/* TODO: Change this timing as given in Datasheet */
/* Timings for MICRON Flash */
#define SLCNAND_MICRON_DA_TIMING EA3250_SLC_TIMING
#define MLCNAND_MICRON_DA_TIMING EA3250_SLC_TIMING

/* Timings for SAMSUNG flash */
#define SLCNAND_SAMSUNG_F1_TIMING EA3250_MLC_TIMING
#define MLCNAND_SAMSUNG_F1_TIMING EA3250_MLC_TIMING

/* List of all types of NAND flash that could be mounted
 * in an EA3250 Board
 */
static struct nand_ids ea3250_nand_ids[] =
{
  /* MICRON Flash */
  {
    LPCNAND_VENDOR_MICRON,
    0xDA, /* Device ID */
    2048, /* Page size */
    64, /* Pages per block */
    2048, /* Total blocks */
	{SLCNAND_MICRON_DA_TIMING, MLCNAND_MICRON_DA_TIMING}/* Timing */
  },

  /* SAMSUNG Flash */
  {
    LPCNAND_VENDOR_SAMSUNG,
    0xF1, /* Device ID */
    2048, /* Page size */
    64, /* Pages per block */
    1024, /* Total blocks */
	{SLCNAND_SAMSUNG_F1_TIMING, MLCNAND_SAMSUNG_F1_TIMING}/* Timing */
  },
};

static struct nand_devinfo_list ea3250_nand_list =
{
  sizeof(ea3250_nand_ids)/sizeof(ea3250_nand_ids[0]),
  ea3250_nand_ids,
  NULL
};

/***********************************************************************
 *
 * Function: slcnand_board_init
 *
 * Purpose: Intializes the SLC NAND contoller and the listed NAND Device
 *
 * Processing:
 *     See function.
 *
 * Parameters: None
 *
 * Outputs: None
 *
 * Returns: Device ID on success
 *          0 on Failure
 *
 * Notes: None
 *
 **********************************************************************/
INT_32 slcnand_board_init(void)
{
  return slcnand_open(SLCNAND, (INT_32)&ea3250_nand_list);
}

/***********************************************************************
 *
 * Function: mlcnand_board_init
 *
 * Purpose: Intializes the MLC NAND contoller and the listed NAND Device
 *
 * Processing:
 *     See function.
 *
 * Parameters: None
 *
 * Outputs: None
 *
 * Returns: Device ID on success
 *          0 on Failure
 *
 * Notes: None
 *
 **********************************************************************/
INT_32 mlcnand_board_init(void)
{
  return mlcnand_open(MLCNAND, (INT_32)&ea3250_nand_list);
}
